@@ -1,6 +1,6 @@
 package Reddit::Client;
 
-our $VERSION = '1.06';
+our $VERSION = '1.07';
 $VERSION = eval $VERSION;
 
 use strict;
@@ -71,9 +71,10 @@ use constant API_COMMENTS	=> 17;
 use constant API_MESSAGES	=> 18;
 use constant API_MARK_READ	=> 19;
 use constant API_MARKALL	=> 20;
+use constant API_MY_SUBREDDITS  => 21;
 
 use constant SUBREDDITS_HOME    => '';
-use constant SUBREDDITS_MINE    => 'mine';
+use constant SUBREDDITS_MINE    => 'subscriber';
 use constant SUBREDDITS_POPULAR => 'popular';
 use constant SUBREDDITS_NEW     => 'new';
 use constant SUBREDDITS_CONTRIB => 'contributor';
@@ -83,10 +84,12 @@ use constant SUBREDDITS_MOD     => 'moderator';
 # Parameters
 #===============================================================================
 
-our $DEBUG           = 0;
-our $BASE_URL        = 'https://oauth.reddit.com';
-our $LINK_URL        = 'https://www.reddit.com';
-our $UA              = sprintf 'Reddit::Client/%f', $VERSION;
+our $DEBUG            = 0;
+our $BASE_URL         = 'https://oauth.reddit.com';
+use constant BASE_URL => 'https://oauth.reddit.com';
+our $LINK_URL         = 'https://www.reddit.com';
+use constant LINK_URL => 'https://www.reddit.com';
+our $UA               = sprintf 'Reddit::Client/%f', $VERSION;
 
 our @API;
 $API[API_ME            ] = ['GET',  '/api/v1/me'     ];
@@ -100,15 +103,16 @@ $API[API_SAVE          ] = ['POST', '/api/save'      ];
 $API[API_UNSAVE        ] = ['POST', '/api/unsave'    ];
 $API[API_HIDE          ] = ['POST', '/api/hide'      ];
 $API[API_UNHIDE        ] = ['POST', '/api/unhide'    ];
-$API[API_SUBREDDITS    ] = ['GET',  '/reddits/%s'    ];
-$API[API_LINKS_OTHER   ] = ['GET',  '/%s'            ];
-$API[API_LINKS_FRONT   ] = ['GET',  '/r/%s/%s'       ];
-$API[API_DEL           ] = ['POST', '/api/del'       ];
-$API[API_MESSAGE       ] = ['POST', '/api/compose'   ];
-$API[API_COMMENTS      ] = ['GET',  '/r/%s/comments' ];
-$API[API_COMMENTS_FRONT] = ['GET',  '/comments'      ];
-$API[API_MESSAGES      ] = ['GET',  '/message/%s'    ];
-$API[API_MARK_READ     ] = ['POST', '/api/read_message'];
+$API[API_SUBREDDITS    ] = ['GET',  '/subreddits/%s' ];
+$API[API_MY_SUBREDDITS ] = ['GET',  '/subreddits/mine/%s'   ];
+$API[API_LINKS_OTHER   ] = ['GET',  '/%s'                   ];
+$API[API_LINKS_FRONT   ] = ['GET',  '/r/%s/%s'              ];
+$API[API_DEL           ] = ['POST', '/api/del'              ];
+$API[API_MESSAGE       ] = ['POST', '/api/compose'          ];
+$API[API_COMMENTS      ] = ['GET',  '/r/%s/comments'        ];
+$API[API_COMMENTS_FRONT] = ['GET',  '/comments'             ];
+$API[API_MESSAGES      ] = ['GET',  '/message/%s'           ];
+$API[API_MARK_READ     ] = ['POST', '/api/read_message'     ];
 $API[API_MARKALL       ] = ['POST', '/api/read_all_messages'];
 #===============================================================================
 # Package routines
@@ -341,23 +345,34 @@ sub me {
 }
 
 sub list_subreddits {
-    my ($self, $type) = @_;
+    	my ($self, %param) = @_;
+
+	my $type = $param{view} || croak "expected view";
+    	my $query  = {};
+        $query->{before} = $param{before} if $param{before};
+        $query->{after}  = $param{after}  if $param{after};
+	if (exists $param{limit}) { $query->{limit} = $param{limit} || 500; }
+	else 			  { $query->{limit} = DEFAULT_LIMIT;	    }
     DEBUG('List subreddits [%s]', $type);
     defined $type || croak 'Expected $type"';
+	my $api = $type eq SUBREDDITS_MOD || $type eq SUBREDDITS_CONTRIB || $type eq SUBREDDITS_MINE ? API_MY_SUBREDDITS : API_SUBREDDITS; 
 
-    my $result = $self->api_json_request(api => API_SUBREDDITS, args => [$type]);
-    return {
-        map { $_->{data}{display_name} => Reddit::Client::SubReddit->new($self, $_->{data}) }
-            @{$result->{data}{children}}
-    };
+    my $result = $self->api_json_request(
+	api => $api, 
+	args => [$type],
+	data => $query,
+    );
+    return [
+        map {Reddit::Client::SubReddit->new($self, $_->{data})} @{$result->{data}{children}}
+    ];
 }
 
-sub mod_subreddits     { return $_[0]->list_subreddits(SUBREDDITS_MOD)     }
-sub my_subreddits      { return $_[0]->list_subreddits(SUBREDDITS_MINE)    }
-sub contrib_subreddits { return $_[0]->list_subreddits(SUBREDDITS_CONTRIB) }
-sub home_subreddits    { return $_[0]->list_subreddits(SUBREDDITS_HOME)    }
-sub popular_subreddits { return $_[0]->list_subreddits(SUBREDDITS_POPULAR) }
-sub new_subreddits     { return $_[0]->list_subreddits(SUBREDDITS_NEW)     }
+sub contrib_subreddits { return $_[0]->list_subreddits(view=>SUBREDDITS_CONTRIB) }
+sub home_subreddits    { return $_[0]->list_subreddits(view=>SUBREDDITS_HOME)    }
+sub mod_subreddits     { return $_[0]->list_subreddits(view=>SUBREDDITS_MOD)     }
+sub my_subreddits      { return $_[0]->list_subreddits(view=>SUBREDDITS_MINE)    }
+sub new_subreddits     { return $_[0]->list_subreddits(view=>SUBREDDITS_NEW)     }
+sub popular_subreddits { return $_[0]->list_subreddits(view=>SUBREDDITS_POPULAR) }
 
 #===============================================================================
 # Inbox and messages
@@ -365,7 +380,8 @@ sub new_subreddits     { return $_[0]->list_subreddits(SUBREDDITS_NEW)     }
 sub get_inbox {
 	my ($self, %param) = @_;
     	my $limit     	= $param{limit}		|| DEFAULT_LIMIT;
-	my $mode	= $param{mode}		|| 'inbox';	
+	my $mode	= $param{mode}		|| MESSAGES_INBOX;	
+	my $view	= $param{view}		|| MESSAGES_INBOX;
 
 	# this before and after business is stupid and needs to be fixed
 	# in 3 separate places
@@ -379,14 +395,13 @@ sub get_inbox {
 	
 	my $result = $self->api_json_request(
 		api	=> API_MESSAGES,
-		args	=> [$mode],
+		args	=> [$view],
 		data	=> $query,
 	);
 
 	return [
 		map { Reddit::Client::Message->new($self, $_->{data}) } @{$result->{data}{children}}
 	];
-        #map {Reddit::Client::Link->new($self, $_->{data})} @{$result->{data}{children}} 
 }
 
 # TODO
@@ -395,7 +410,7 @@ sub mark_read {
 
 }
 
-sub mark_all_read {
+sub mark_inbox_read {
 	my $self = shift;
 	my ($method, $path) = @{$API[API_MARKALL]};
 	my $post_data = {api_type=>'json'};
@@ -417,17 +432,15 @@ sub info {
 	args => [$id], 
 	data=>$query);
     my $rtn = $info->{data}->{children}[0]->{data};
-    $rtn->{kind} = $info->{data}->{children}[0]->{kind};
+    $rtn->{kind} = $info->{data}->{children}[0]->{kind} if $rtn;
     return $rtn;
 }
 
-# shortcut to get permalink from info() object
 sub get_permalink {
-	# the naming convention is inconsistent here; a comment "id" has no 
-	# prefix, and its "name" is the id prefixed with "t1_". All other ids 
-	# have a prefix.
-	my ($self, $commentid, $link_id) = @_;
-	my $info = $self->info($link_id);
+	# commentid is short id
+	# post_fullname is because we need to get the post to get its permalink
+	my ($self, $commentid, $post_fullname) = @_;
+	my $info = $self->info($post_fullname);
 	return sprintf "%s%s%s", $LINK_URL, $info->{permalink}, $commentid;
 }
 
@@ -446,11 +459,11 @@ sub fetch_links {
     my ($self, %param) = @_;
     my $subreddit = $param{subreddit} || '';
     my $view      = $param{view}      || VIEW_DEFAULT;
-    my $limit     = $param{limit}     || DEFAULT_LIMIT;
-    my $before    = $param{before};
-    my $after     = $param{after};
+    #my $limit     = $param{limit}     || DEFAULT_LIMIT;
+    #my $before    = $param{before};
+    #my $after     = $param{after};
 
-    DEBUG('Fetch %d link(s): %s/%s?before=%s&after=%s', $limit, $subreddit, $view, ($before || '-'), ($after || '-'));
+    #DEBUG('Fetch %d link(s): %s/%s?before=%s&after=%s', $limit, $subreddit, $view, ($before || '-'), ($after || '-'));
 
     	my $query  = {};
         $query->{before} = $param{before} if $param{before};
@@ -475,6 +488,38 @@ sub fetch_links {
     ];
 }
 
+# alias for fetch_links to make naming convention consistent
+sub get_links {
+    	my ($self, %param) = @_;
+	return $self->fetch_links(%param);
+}
+
+sub get_link {
+    	my ($self, $fullname) = @_;
+	if (	!$fullname || 
+		length($fullname) < 8 ||
+		substr($fullname, 0, 2) ne "t3")
+	{ croak "expected link fullname (i.e. t3_3ng7r5)";	}
+
+	my $info = $self->info($fullname);
+
+	return $info if !$info;
+	return Reddit::Client::Link->new($self, $info);
+}
+
+sub get_comment {
+    	my ($self, $fullname) = @_;
+	if (	!$fullname || 
+		length($fullname) < 8 ||
+		substr($fullname, 0, 2) ne "t1")
+	{ croak "expected comment fullname (i.e. t1_cvp5afk)"; }
+
+	my $info = $self->info($fullname);
+
+	return $info if !$info;
+	return Reddit::Client::Comment->new($self, $info);
+}
+
 sub get_subreddit_comments {
 	my ($self, %param) = @_;
 	my $subreddit 	= $param{subreddit} 	|| '';
@@ -511,10 +556,11 @@ sub get_subreddit_comments {
 #===============================================================================
 
 # TODO unit test
-sub delete_item {
-    my ($self, %param) = @_;
-    my $name = $param{name} || croak 'Expected "name"';
-    $self->require_login;
+sub delete {
+    my ($self, $name) = @_;
+    croak 'Expected $fullname' if !$name;
+    my $type = substr $name, 0, 2;
+    croak '$fullname must be a post or comment' if $type ne 't1' && $type ne 't3';
 
     DEBUG('Delete post/comment %s', $name);
 
@@ -590,9 +636,9 @@ sub submit_comment {
     my $parent_id = $param{parent_id} || croak 'Expected "parent_id"';
     my $comment   = $param{text}      || croak 'Expected "text"';
 
+    croak '$fullname must be a post or comment' if !$self->ispost($parent_id) && !$self->iscomment($parent_id);
     DEBUG('Submit comment under %s', $parent_id);
 
-    $self->require_login;
     my $result = $self->api_json_request(api => API_COMMENT, data => {
         thing_id => $parent_id,
         text     => $comment,
@@ -634,8 +680,9 @@ sub vote {
     my ($self, $name, $direction) = @_;
     defined $name      || croak 'Expected $name';
     defined $direction || croak 'Expected $direction';
-    DEBUG('Vote %d for %s', $direction, $name);
+    croak '$fullname must be a post or comment' if !$self->ispost($name) && !$self->iscomment($name);
     croak 'Invalid vote direction' unless "$direction" =~ /^(-1|0|1)$/;
+    DEBUG('Vote %d for %s', $direction, $name);
     $self->api_json_request(api => API_VOTE, data => { dir => $direction, id  => $name });
 }
 
@@ -645,32 +692,51 @@ sub vote {
 
 sub save {
     my $self = shift;
-    my $name = shift || croak 'Expected $name';
+    my $name = shift || croak 'Expected $fullname';
+    croak '$fullname must be a post or comment' if !$self->ispost($name) && !$self->iscomment($name);
     DEBUG('Save %s', $name);
     $self->api_json_request(api => API_SAVE, data => { id => $name });
 }
 
 sub unsave {
     my $self = shift;
-    my $name = shift || croak 'Expected $name';
+    my $name = shift || croak 'Expected $fullname';
+    croak '$fullname must be a post or comment' if !$self->ispost($name) && !$self->iscomment($name);
     DEBUG('Unsave %s', $name);
     $self->api_json_request(api => API_UNSAVE, data => { id => $name });
 }
 
 sub hide {
     my $self = shift;
-    my $name = shift || croak 'Expected $name';
+    my $name = shift || croak 'Expected $fullname';
+    croak '$fullname must be a post' if !$self->ispost($name);
     DEBUG('Hide %s', $name);
     $self->api_json_request(api => API_HIDE, data => { id => $name });
 }
 
 sub unhide {
     my $self = shift;
-    my $name = shift || croak 'Expected $name';
+    my $name = shift || croak 'Expected $fullname';
+    croak '$fullname must be a post' if !$self->ispost($name);
     DEBUG('Unhide %s', $name);
     $self->api_json_request(api => API_UNHIDE, data => { id => $name });
 }
 
+
+# Internal
+
+sub ispost {
+	my ($self, $name) = @_;
+    	my $type = substr $name, 0, 2;
+	return $type eq 't3';
+}
+
+sub iscomment {
+	my ($self, $name) = @_;
+    	my $type = substr $name, 0, 2;
+	return $type eq 't1';
+}
+
 1;
 
 __END__
@@ -681,6 +747,8 @@ __END__
 
 Reddit::Client - A Perl wrapper for the Reddit API.
 
+The official documentation can be found at L<http://redditclient.readthedocs.org/en/latest/>.
+
 =head1 SYNOPSIS
 
     use Reddit::Client;
@@ -826,8 +894,7 @@ Mark your inbox read. May not happen instantly.
 
 =item info($fullname)
 
-Returns a hash of information about C<$item_id>, which must be a fullname
-(i.e. the combination of a thing's ID and its prefix: "t3_xxxxx". A list of prefixes can be found at L<https://www.reddit.com/dev/api>). 
+Returns a hash of extended information about C<$fullname>. Contains some information not available elsewhere.
 
 
 =item list_subreddits($type)
@@ -888,7 +955,7 @@ slicing the feed up, just as the reddit urls themselves do. Data is returned
 as a hash with three keys, I<before>, I<after>, and I<items>.
 
 
-=item delete_item(name => ...)
+=item delete(name => ...)
 
 Deletes a post or comment. The object's fullname is required.
 
