@@ -18,6 +18,16 @@ sub comments {
     return $self->{session}->get_comments(permalink => $self->{permalink});
 }
 
+sub hide {
+    my $self = shift;
+    $self->{session}->hide($self->{name});
+}
+
+sub unhide {
+    my $self = shift;
+    $self->{session}->unhide($self->{name});
+}
+
 1;
 
 __END__
